/*
- 파일명: my-toc-frontend.js
- 위치: /my-toc-plugin/assets/js/
- 기능: TOC 플러그인의 프론트엔드 동작을 관리하는 자바스크립트
- 작성일: 2025-03-23
- 수정일: 2025-03-25
*/

// ===================================
// 문서 준비 및 주요 기능 초기화
// ===================================
jQuery(document).ready(function($) {
    // 기능 초기화
    initTocToggle();
    initSmoothScroll();
    setupStickyToc();
    adjustStickyPosition();
    initHighlighting();
    removeAllBorders();
    applyCustomColors();
    
    // 이벤트 리스너 설정
    $(window).on('resize', adjustStickyPosition);
    
    // ===================================
    // 토글 기능
    // ===================================
    function initTocToggle() {
        $('.my-toc-toggle').on('click', function() {
            var $toc = $(this).closest('.my-toc');
            var $content = $toc.find('.my-toc-content');
            var $icon = $(this).find('i');
            
            $content.toggleClass('my-toc-content-hidden');
            
            if ($content.hasClass('my-toc-content-hidden')) {
                $icon.removeClass('bi-dash').addClass('bi-plus');
                $toc.addClass('my-toc-collapsed');
            } else {
                $icon.removeClass('bi-plus').addClass('bi-dash');
                $toc.removeClass('my-toc-collapsed');
            }
        });
    }
    
    // ===================================
// 스크롤 하이라이팅
// ===================================
function initHighlighting() {
    // 초기 실행
    setTimeout(refreshHeaderHighlighting, 500);
    
    // 스크롤 이벤트에 단순화된 접근 방식 적용
    $(window).on('scroll', function() {
        refreshHeaderHighlighting();
    });
    
    // 창 크기 변경 시에도 업데이트
    $(window).on('resize', function() {
        refreshHeaderHighlighting();
    });
}

function refreshHeaderHighlighting() {
    // 화면 높이와 스크롤 위치 가져오기
    const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
    const windowHeight = window.innerHeight;
    
    // 모든 헤더 요소 가져오기
    const headers = document.querySelectorAll('h2[id^="toc-"], h3[id^="toc-"], h4[id^="toc-"]');
    if (headers.length === 0) return;
    
    let bestHeader = null;
    let bestPosition = -Infinity;
    
    // 화면 내에 있는 헤더들 검사
    for (let i = 0; i < headers.length; i++) {
        const header = headers[i];
        const rect = header.getBoundingClientRect();
        const headerTop = rect.top + scrollTop;
        
        // 뷰포트 상단부터 100px 범위 내에 있는 헤더 중 
        // 가장 위에 있는 헤더 선택
        if ((headerTop <= scrollTop + 100) && headerTop > bestPosition) {
            bestPosition = headerTop;
            bestHeader = header;
        }
    }
    
    // 적합한 헤더가 없으면 화면 내에서 첫 번째 보이는 헤더 선택
    if (!bestHeader) {
        for (let i = 0; i < headers.length; i++) {
            const header = headers[i];
            const rect = header.getBoundingClientRect();
            if (rect.top >= 0 && rect.top <= windowHeight) {
                bestHeader = header;
                break;
            }
        }
    }
    
    // 적합한 헤더가 없고 스크롤이 거의 상단에 있으면 첫 번째 헤더 선택
    if (!bestHeader && scrollTop < 50 && headers.length > 0) {
        bestHeader = headers[0];
    }
    
    // 적합한 헤더가 없고 스크롤이 거의 하단에 있으면 마지막 헤더 선택
    if (!bestHeader && scrollTop + windowHeight >= document.body.scrollHeight - 50) {
        bestHeader = headers[headers.length - 1];
    }
    
    // 선택된 헤더가 있으면 하이라이팅
    if (bestHeader) {
        highlightTocItem(bestHeader.id);
    }
}

function highlightTocItem(headerId) {
    if (!headerId) return;
    
    // 모든 기존 스타일 및 클래스 초기화
    $('.my-toc-item-active').removeClass('my-toc-item-active');
    $('.my-toc-list a, .my-toc-sublist a').removeClass('active-link');
    $('.my-toc-list a, .my-toc-sublist a').each(function() {
        this.removeAttribute('style');
    });
    
    // 새 항목 활성화
    const $activeLink = $('.my-toc-list a[href="#' + headerId + '"], .my-toc-sublist a[href="#' + headerId + '"]');
    
    if ($activeLink.length === 0) {
        return;
    }
    
    $activeLink.addClass('active-link');
    $activeLink.parent().addClass('my-toc-item-active');
    
    // 인라인 스타일 강제 적용
    $activeLink.each(function() {
        // DOM 요소 직접 접근
        const element = this;
        
        // 테마 색상 결정
        const $toc = $(element).closest('.my-toc');
        let borderColor = '#0073aa'; // 기본값
        
        if ($toc.hasClass('my-toc-theme-blue')) {
            borderColor = '#0066cc';
        } else if ($toc.hasClass('my-toc-theme-white')) {
            borderColor = '#2271b1';
        } else if ($toc.hasClass('my-toc-theme-warm')) {
            borderColor = '#e65100';
        }
        
        // 스타일 적용
        element.style.setProperty('font-weight', 'bold', 'important');
        element.style.setProperty('background-color', 'rgba(0,0,0,0.05)', 'important');
        element.style.setProperty('padding-left', '6px', 'important');
        element.style.setProperty('border-left', '3px solid ' + borderColor, 'important');
        element.style.setProperty('text-decoration', 'none', 'important');
        element.style.setProperty('background-image', 'none', 'important');
        element.style.setProperty('background-size', 'auto', 'important');
        element.style.setProperty('background-position', 'initial', 'important');
        element.style.setProperty('color', '#333', 'important');
        element.style.setProperty('opacity', '1', 'important');
        element.style.setProperty('visibility', 'visible', 'important');
        element.style.setProperty('border-radius', '0', 'important');
        element.style.setProperty('transition', 'none', 'important');
        element.style.setProperty('box-shadow', 'none', 'important');
        element.style.setProperty('outline', 'none', 'important');
        element.style.setProperty('display', 'block', 'important');
    });
}



    
    // ===================================
    // 부드러운 스크롤
    // ===================================
    function initSmoothScroll() {
        $(document).on('click', '.my-toc-list a, .my-toc-sublist a', function(e) {
            e.preventDefault();
            
            var targetId = $(this).attr('href').replace('#', '');
            var element = document.getElementById(targetId);
            
            if (element) {
                // setTimeout으로 현재 이벤트 루프 이후에 실행
                setTimeout(function() {
                    window.scrollTo(0, element.offsetTop - 60);
                }, 10);
            }
        });
    }
    
    // ===================================
    // 붙박이 기능
    // ===================================
    function setupStickyToc() {
        if (typeof myTocOptions === 'undefined') return;
        
        // 붙박이 기능 상태에 따라 처리
        if (myTocOptions.sticky === 'on') {
            $('.my-toc').addClass('my-toc-sticky');
            $('.my-toc').addClass('my-toc-sticky-' + myTocOptions.stickyPosition);
        } else {
            // 붙박이 기능이 off일 경우 관련 클래스 제거
            $('.my-toc').removeClass('my-toc-sticky');
            $('.my-toc').removeClass(function(index, className) {
                return (className.match(/(^|\s)my-toc-sticky-\S+/g) || []).join(' ');
            });
        }
    }
    
    // ===================================
    // 반응형 처리
    // ===================================
    function adjustStickyPosition() {
        if ($('.my-toc-sticky').length) {
            var windowWidth = $(window).width();
            
            if (windowWidth <= 768) {
                // 모바일 화면에서는 하단에 표시
                $('.my-toc-sticky').css({
                    'bottom': '10px',
                    'top': 'auto',
                    'transform': 'translateX(-50%)'
                });
            } else {
                // 원래 위치로 복원
                $('.my-toc-sticky').css({
                    'bottom': '',
                    'top': '',
                    'transform': ''
                });
                
                // 중간 위치인 경우 transform 복원
                if ($('.my-toc-sticky-left-middle, .my-toc-sticky-right-middle').length) {
                    $('.my-toc-sticky-left-middle, .my-toc-sticky-right-middle').css({
                        'transform': 'translateY(-50%)'
                    });
                }
            }
        }
    }
    
    // ===================================
    // 테두리 제거
    // ===================================
    function removeAllBorders() {
        // 모든 목차 링크에서 테두리 제거
        $('.my-toc-list a, .my-toc-sublist a, .my-toc-item a').css({
            'border': 'none',
            'box-shadow': 'none',
            'outline': 'none'
        });
        
        // 인라인 스타일로 강제 적용
        $('.my-toc-list a, .my-toc-sublist a, .my-toc-item a').each(function() {
            this.style.setProperty('border', 'none', 'important');
            this.style.setProperty('box-shadow', 'none', 'important');
            this.style.setProperty('outline', 'none', 'important');
        });
    }
    
    // ===================================
    // 사용자 정의 색상 적용
    // ===================================
    function applyCustomColors() {
        $('.my-toc-theme-custom').each(function() {
            var $toc = $(this);
            var linkColor = $toc.attr('data-link-color');
            var hoverColor = $toc.attr('data-hover-color');
            var visitedColor = $toc.attr('data-visited-color');
            
            if (linkColor) {
                // 링크 색상 직접 적용
                $toc.find('.my-toc-list a, .my-toc-sublist a').css('color', linkColor);
                
                // 인라인 스타일 추가
                $toc.find('.my-toc-list a, .my-toc-sublist a').each(function() {
                    this.style.setProperty('color', linkColor, 'important');
                });
            }
            
            // 호버 이벤트 처리
            if (hoverColor) {
                $toc.find('.my-toc-list a, .my-toc-sublist a').off('mouseenter mouseleave')
                    .on('mouseenter', function() {
                        $(this).css({
                            'color': hoverColor,
                            'background-color': 'rgba(0,0,0,0.1)',
                            'font-weight': 'bold',
                            'text-shadow': '0 0 0.5px currentColor',
                            'border': 'none',
                            'box-shadow': 'none'
                        });
                        
                        // 인라인 스타일 추가
                        this.style.setProperty('color', hoverColor, 'important');
                        this.style.setProperty('background-color', 'rgba(0,0,0,0.1)', 'important');
                        this.style.setProperty('border', 'none', 'important');
                    })
                    .on('mouseleave', function() {
                        $(this).css({
                            'color': linkColor || '',
                            'background-color': '',
                            'font-weight': '',
                            'text-shadow': '',
                            'border': 'none',
                            'box-shadow': 'none'
                        });
                        
                        // 인라인 스타일 원복
                        if (linkColor) {
                            this.style.setProperty('color', linkColor, 'important');
                        } else {
                            this.style.removeProperty('color');
                        }
                        this.style.removeProperty('background-color');
                        this.style.setProperty('border', 'none', 'important');
                    });
            }
            
            // 방문한 링크 스타일
            if (visitedColor) {
                $toc.find('.my-toc-list a:visited, .my-toc-sublist a:visited').css('color', visitedColor);
                
                // 인라인 스타일 추가
                $toc.find('.my-toc-list a:visited, .my-toc-sublist a:visited').each(function() {
                    this.style.setProperty('color', visitedColor, 'important');
                });
            }
        });
        
        // DOM 변경 감지하여 색상 적용 유지
        setupMutationObserver();
    }
    
    // ===================================
    // DOM 변경 감지
    // ===================================
    function setupMutationObserver() {
        // 이미 설정된 observer가 있으면 중복 설정 방지
        if (window.myTocObserver) return;
        
        // DOM 변경 감지하여 재실행
        window.myTocObserver = new MutationObserver(function(mutations) {
            applyCustomColors();
            removeAllBorders();
        });
        
        // 관찰 대상 및 설정
        var target = document.querySelector('body');
        if (target) {
            window.myTocObserver.observe(target, { childList: true, subtree: true });
        }
    }
});