<?php
/*
- 파일명: class-my-toc-admin.php
- 위치: /my-toc-plugin/includes/
- 기능: TOC 플러그인의 관리자 설정 페이지를 처리하는 클래스
- 작성일: 2025-03-23
*/

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

/**
 * TOC 플러그인의 관리자 기능을 담당하는 클래스
 */
class My_TOC_Admin {
    
    /**
     * 클래스 초기화
     *
     * @return void
     */
    public function init() {
        // 관리자 메뉴 추가
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        // 관리자 스크립트 및 스타일 등록
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        
        // 설정 등록
        add_action('admin_init', array($this, 'register_settings'));
        
        // 메타박스 추가 (포스트별 TOC 비활성화 옵션)
        add_action('add_meta_boxes', array($this, 'add_toc_meta_box'));
        
        // 메타박스 저장
        add_action('save_post', array($this, 'save_toc_meta_box'));
    }
    
    /**
     * 관리자 메뉴 추가
     */
    public function add_admin_menu() {
        add_menu_page(
            '내 목차 설정',          // 페이지 제목
            '내 목차',              // 메뉴 제목
            'manage_options',      // 필요한 권한
            'my-toc-settings',     // 메뉴 슬러그
            array($this, 'display_settings_page'), // 콜백 함수
            'dashicons-list-view', // 아이콘
            100                    // 위치
        );
    }
    
    /**
     * 관리자 설정 페이지 표시
     */
    public function display_settings_page() {
        ?>
        <div class="wrap my-toc-admin-wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            
            <form method="post" action="options.php">
                <?php
                // 설정 필드 출력
                settings_fields('my_toc_options_group');
                do_settings_sections('my-toc-settings');
                submit_button('설정 저장');
                ?>
            </form>
        </div>
        <?php
    }
    
    /**
     * 설정 필드 등록
     */
    public function register_settings() {
        // 설정 그룹 등록
        register_setting(
            'my_toc_options_group',     // 옵션 그룹
            'my_toc_options',           // 옵션 이름
            array($this, 'sanitize_options') // 유효성 검사 콜백
        );
        
        // 기본 설정 섹션
        $this->register_general_settings();
        
        // 스타일 설정 섹션
        $this->register_style_settings();
        
        // 고급 설정 섹션
        $this->register_advanced_settings();
    }
    
    /**
     * 기본 설정 섹션 등록
     */
    private function register_general_settings() {
        add_settings_section(
            'my_toc_general_section',   // ID
            '기본 설정',                // 제목
            array($this, 'general_section_callback'), // 콜백
            'my-toc-settings'           // 페이지
        );
        
        // 위치 필드
        add_settings_field(
            'position',                  // ID
            '목차 위치',                 // 제목
            array($this, 'position_callback'), // 콜백
            'my-toc-settings',           // 페이지
            'my_toc_general_section'     // 섹션
        );
        
        // 표시 조건 필드
        add_settings_field(
            'min_headers', 
            '표시 조건', 
            array($this, 'min_headers_callback'), 
            'my-toc-settings', 
            'my_toc_general_section'
        );
        
        // 헤더 라벨 필드
        add_settings_field(
            'header_label', 
            '헤더 라벨', 
            array($this, 'header_label_callback'), 
            'my-toc-settings', 
            'my_toc_general_section'
        );
        
        // 초기 보기 상태 필드
        add_settings_field(
            'initial_view', 
            '초기 보기 상태', 
            array($this, 'initial_view_callback'), 
            'my-toc-settings', 
            'my_toc_general_section'
        );
    }
    
    /**
     * 스타일 설정 섹션 등록
     */
    private function register_style_settings() {
        add_settings_section(
            'my_toc_style_section', 
            '스타일 설정', 
            array($this, 'style_section_callback'), 
            'my-toc-settings'
        );
        
        // 테마 필드
        add_settings_field(
            'theme', 
            '테마', 
            array($this, 'theme_callback'), 
            'my-toc-settings', 
            'my_toc_style_section'
        );
        
        // 사용자 정의 색상 필드
        add_settings_field(
            'custom_colors', 
            '사용자 정의 색상', 
            array($this, 'custom_colors_callback'), 
            'my-toc-settings', 
            'my_toc_style_section'
        );
        
        // 목차 기호 필드
        add_settings_field(
            'prefix_markers', 
            '목차 기호', 
            array($this, 'prefix_markers_callback'), 
            'my-toc-settings', 
            'my_toc_style_section'
        );
    }
    
    /**
     * 고급 설정 섹션 등록
     */
    private function register_advanced_settings() {
        add_settings_section(
            'my_toc_advanced_section', 
            '고급 설정', 
            array($this, 'advanced_section_callback'), 
            'my-toc-settings'
        );
        
        // 붙박이 기능 필드
        add_settings_field(
            'sticky', 
            '붙박이 기능', 
            array($this, 'sticky_callback'), 
            'my-toc-settings', 
            'my_toc_advanced_section'
        );
        
        // 붙박이 위치 필드
        add_settings_field(
            'sticky_position', 
            '붙박이 위치', 
            array($this, 'sticky_position_callback'), 
            'my-toc-settings', 
            'my_toc_advanced_section'
        );
    }
    
    /**
	 * 섹션 콜백 함수들
	 */
	public function general_section_callback() {
		// 섹션 설명만 출력 (제목은 WordPress가 자동으로 출력)
		echo '<p>목차의 기본 표시 방식을 설정합니다.</p>';
	}

	public function style_section_callback() {
		echo '<p>목차의 스타일과 디자인을 설정합니다.</p>';
	}

	public function advanced_section_callback() {
		echo '<p>고급 목차 기능을 설정합니다.</p>';
	}
    
    /**
     * 위치 필드 콜백
     */
    public function position_callback() {
        $options = get_option('my_toc_options');
        $position = isset($options['position']) ? $options['position'] : 'before';
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-arrows-move text-primary"></i>
            </span>
            <select name="my_toc_options[position]" class="form-control border-start-0" placeholder="목차 위치를 선택하세요">
                <option value="before" <?php selected($position, 'before'); ?>>첫 머리글(H2) 앞 (기본값)</option>
                <option value="after" <?php selected($position, 'after'); ?>>첫 머리글(H2) 뒤</option>
            </select>
        </div>
        <p class="description">목차가 표시될 위치를 선택합니다.</p>
        <?php
    }
    
    /**
     * 표시 조건 필드 콜백
     */
    public function min_headers_callback() {
        $options = get_option('my_toc_options');
        $min_headers = isset($options['min_headers']) ? $options['min_headers'] : 2;
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-list-ul text-primary"></i>
            </span>
            <input type="number" name="my_toc_options[min_headers]" min="1" max="10" value="<?php echo esc_attr($min_headers); ?>" class="form-control border-start-0" placeholder="필요한 최소 헤더 수">
        </div>
        <p class="description">H2 태그가 이 숫자 이상일 때만 목차가 표시됩니다. (최소 2 권장)</p>
        <?php
    }
    
    /**
     * 헤더 라벨 필드 콜백
     */
    public function header_label_callback() {
        $options = get_option('my_toc_options');
        $header_label = isset($options['header_label']) ? $options['header_label'] : '목차';
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-type-h1 text-primary"></i>
            </span>
            <input type="text" name="my_toc_options[header_label]" value="<?php echo esc_attr($header_label); ?>" class="form-control border-start-0" placeholder="목차 상단에 표시할 제목">
        </div>
        <p class="description">목차 상단에 표시될 제목을 입력하세요. (예: '목차', '콘텐츠', '페이지 콘텐츠')</p>
        <?php
    }
    
    /**
     * 목차 기호 필드 콜백
     */
    public function prefix_markers_callback() {
        $options = get_option('my_toc_options');
        $h2_prefix = isset($options['h2_prefix']) ? $options['h2_prefix'] : '';
        $h3_prefix = isset($options['h3_prefix']) ? $options['h3_prefix'] : '';
        ?>
        <div>
            <label for="my_toc_options_h2_prefix">H2 앞에 넣을 문구:</label>
            <div class="input-group">
                <span class="input-group-text bg-white border-end-0">
                    <i class="bi bi-list-task text-primary"></i>
                </span>
                <input type="text" name="my_toc_options[h2_prefix]" id="my_toc_options_h2_prefix" value="<?php echo esc_attr($h2_prefix); ?>" class="form-control border-start-0" placeholder="H2 앞에 표시할 기호 입력">
            </div>
			<div class="my-toc-symbol-buttons">
                <button type="button" class="button symbol-btn" data-symbol="• " data-target="h2">• 글머리 기호</button>
                <button type="button" class="button symbol-btn" data-symbol="› " data-target="h2">› 화살표</button>
                <button type="button" class="button symbol-btn" data-symbol="✔ " data-target="h2">✔ 체크</button>
                <button type="button" class="button symbol-btn" data-symbol="+ " data-target="h2">+ 더하기</button>
                <button type="button" class="button symbol-btn" data-symbol="- " data-target="h2">- 빼기</button>
            </div>
            <p class="description">H2 헤더 앞에 표시할 문구를 입력하세요. 비워두면 기호 없이 표시됩니다.</p>
        </div>
        
        <div style="margin-top: 10px;">
            <label for="my_toc_options_h3_prefix">H3 앞에 넣을 문구:</label>
            <div class="input-group">
                <span class="input-group-text bg-white border-end-0">
                    <i class="bi bi-list-nested text-primary"></i>
                </span>
                <input type="text" name="my_toc_options[h3_prefix]" id="my_toc_options_h3_prefix" value="<?php echo esc_attr($h3_prefix); ?>" class="form-control border-start-0" placeholder="H3 앞에 표시할 기호 입력">
            </div>
			<div class="my-toc-symbol-buttons" style="margin-top: 5px;">
                <button type="button" class="button symbol-btn" data-symbol="• " data-target="h3">• 글머리 기호</button>
                <button type="button" class="button symbol-btn" data-symbol="› " data-target="h3">› 화살표</button>
                <button type="button" class="button symbol-btn" data-symbol="✔ " data-target="h3">✔ 체크</button>
                <button type="button" class="button symbol-btn" data-symbol="+ " data-target="h3">+ 더하기</button>
                <button type="button" class="button symbol-btn" data-symbol="- " data-target="h3">- 빼기</button>
            </div>
            <p class="description">H3 헤더 앞에 표시할 문구를 입력하세요. 비워두면 기호 없이 표시됩니다.</p>
        </div>
        
        
        <?php
    }
    
    /**
     * 초기 보기 상태 필드 콜백
     */
    public function initial_view_callback() {
        $options = get_option('my_toc_options');
        $initial_view = isset($options['initial_view']) ? $options['initial_view'] : 'show';
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-eye text-primary"></i>
            </span>
            <select name="my_toc_options[initial_view]" class="form-control border-start-0" placeholder="초기 보기 상태 선택">
                <option value="show" <?php selected($initial_view, 'show'); ?>>표시 (기본값)</option>
                <option value="hide" <?php selected($initial_view, 'hide'); ?>>숨김</option>
            </select>
        </div>
        <p class="description">페이지 로드 시 목차의 초기 표시 상태를 설정합니다.</p>
        <?php
    }
    
    /**
	 * 테마 필드 콜백
	 */
	public function theme_callback() {
		$options = get_option('my_toc_options');
		$theme = isset($options['theme']) ? $options['theme'] : 'gray';
		?>
		<div class="input-group">
			<span class="input-group-text bg-white border-end-0">
				<i class="bi bi-palette text-primary"></i>
			</span>
			<select name="my_toc_options[theme]" id="my-toc-theme" class="form-control border-start-0" placeholder="테마 선택">
				<option value="gray" <?php selected($theme, 'gray'); ?>>회색</option>
				<option value="blue" <?php selected($theme, 'blue'); ?>>하늘색</option>
				<option value="white" <?php selected($theme, 'white'); ?>>하얀색</option>
				<option value="warm" <?php selected($theme, 'warm'); ?>>따뜻한 색</option>
				<option value="custom" <?php selected($theme, 'custom'); ?>>사용자 정의</option>
			</select>
		</div>
		<p class="description">목차의 기본 테마를 선택합니다. '사용자 정의'를 선택하면 아래에서 색상을 직접 설정할 수 있습니다.</p>
		
		<!-- 테마 미리보기 영역 추가 -->
		<div class="my-toc-theme-preview">
			<h3>테마 미리보기</h3>
			<div id="my-toc-live-preview">
				<?php echo $this->generate_preview_html($theme, $options); ?>
			</div>
		</div>
		<?php
	}
		
		
	/**
	 * 미리보기 HTML 생성
	 *
	 * @param string $theme 선택된 테마
	 * @param array $options 플러그인 옵션
	 * @return string 미리보기 HTML
	 */
	private function generate_preview_html($theme, $options) {
		// 기본 옵션 설정
		$header_label = isset($options['header_label']) ? esc_html($options['header_label']) : '목차';
		$initial_view = isset($options['initial_view']) ? $options['initial_view'] : 'show';
		$h2_prefix = isset($options['h2_prefix']) ? $options['h2_prefix'] : '';
		$h3_prefix = isset($options['h3_prefix']) ? $options['h3_prefix'] : '';
		
		// 사용자 정의 색상 데이터 속성 준비
		$data_attrs = '';
		$custom_style = '';
		
		if ($theme == 'custom' && isset($options['custom_colors']) && is_array($options['custom_colors'])) {
			$colors = $options['custom_colors'];
			
			// 인라인 스타일 생성
			$custom_style = '<style>';
			$custom_style .= '#my-toc-live-preview .my-toc.my-toc-theme-custom {';
			
			if (isset($colors['background'])) {
				$custom_style .= 'background-color: ' . esc_attr($colors['background']) . ' !important;';
			}
			
			if (isset($colors['border'])) {
				$custom_style .= 'border-color: ' . esc_attr($colors['border']) . ' !important;';
			}
			
			$custom_style .= '}';
			
			if (isset($colors['title'])) {
				$custom_style .= '#my-toc-live-preview .my-toc.my-toc-theme-custom .my-toc-title {';
				$custom_style .= 'color: ' . esc_attr($colors['title']) . ' !important;';
				$custom_style .= '}';
			}
			
			if (isset($colors['link'])) {
				$custom_style .= '#my-toc-live-preview .my-toc.my-toc-theme-custom .my-toc-list a {';
				$custom_style .= 'color: ' . esc_attr($colors['link']) . ' !important;';
				$custom_style .= '}';
			}
			
			if (isset($colors['hover'])) {
				$custom_style .= '#my-toc-live-preview .my-toc.my-toc-theme-custom .my-toc-list a:hover {';
				$custom_style .= 'color: ' . esc_attr($colors['hover']) . ' !important;';
				$custom_style .= '}';
			}
			
			$custom_style .= '</style>';
			
			// 데이터 속성 설정
			$link_color = isset($colors['link']) ? esc_attr($colors['link']) : '#0073aa';
			$hover_color = isset($colors['hover']) ? esc_attr($colors['hover']) : '#00a0d2';
			$visited_color = isset($colors['visited']) ? esc_attr($colors['visited']) : '#800080';
			
			$data_attrs = ' data-link-color="' . $link_color . '"';
			$data_attrs .= ' data-hover-color="' . $hover_color . '"';
			$data_attrs .= ' data-visited-color="' . $visited_color . '"';
		}
		
		// 목차 HTML 클래스 설정
		$toc_classes = array('my-toc');
		$toc_classes[] = 'my-toc-theme-' . $theme;
		
		if ($initial_view == 'hide') {
			$toc_classes[] = 'my-toc-collapsed';
		}
		
		// 미리보기 HTML 생성
		$html = $custom_style;
		$html .= '<div class="' . esc_attr(implode(' ', $toc_classes)) . '"' . $data_attrs . '>';
		
		// 헤더 및 토글 버튼
		$html .= '<div class="my-toc-header">';
		$html .= '<span class="my-toc-title">' . $header_label . '</span>';
		$html .= '<span class="my-toc-toggle" title="' . esc_attr('토글 목차') . '"><i class="bi ' . ($initial_view == 'hide' ? 'bi-plus' : 'bi-dash') . '"></i></span>';
		$html .= '</div>';
		
		// 목차 콘텐츠
		$html .= '<div class="my-toc-content' . ($initial_view == 'hide' ? ' my-toc-content-hidden' : '') . '">';
		$html .= '<ul class="my-toc-list">';
		
		// 샘플 항목 추가
		$html .= '<li class="my-toc-item my-toc-level-2">';
		$html .= '<a href="javascript:void(0);">';
		if (!empty($h2_prefix)) {
			$html .= '<span class="my-toc-prefix">' . esc_html($h2_prefix) . '</span>';
		}
		$html .= '첫 번째 섹션 제목</a>';
		
		// 하위 항목 추가
		$html .= '<ul class="my-toc-sublist">';
		$html .= '<li class="my-toc-item my-toc-level-3">';
		$html .= '<a href="javascript:void(0);">';
		if (!empty($h3_prefix)) {
			$html .= '<span class="my-toc-prefix">' . esc_html($h3_prefix) . '</span>';
		}
		$html .= '하위 섹션 1.1</a></li>';
		$html .= '<li class="my-toc-item my-toc-level-3">';
		$html .= '<a href="javascript:void(0);">';
		if (!empty($h3_prefix)) {
			$html .= '<span class="my-toc-prefix">' . esc_html($h3_prefix) . '</span>';
		}
		$html .= '하위 섹션 1.2</a></li>';
		$html .= '</ul>';
		$html .= '</li>';
		
		// 두 번째 섹션
		$html .= '<li class="my-toc-item my-toc-level-2">';
		$html .= '<a href="javascript:void(0);" class="active-link">';
		if (!empty($h2_prefix)) {
			$html .= '<span class="my-toc-prefix">' . esc_html($h2_prefix) . '</span>';
		}
		$html .= '두 번째 섹션 제목 (현재 위치)</a></li>';
		
		// 세 번째 섹션
		$html .= '<li class="my-toc-item my-toc-level-2">';
		$html .= '<a href="javascript:void(0);">';
		if (!empty($h2_prefix)) {
			$html .= '<span class="my-toc-prefix">' . esc_html($h2_prefix) . '</span>';
		}
		$html .= '세 번째 섹션 제목</a></li>';
		
		$html .= '</ul>'; // .my-toc-list 닫기
		$html .= '</div>'; // .my-toc-content 닫기
		$html .= '</div>'; // .my-toc 닫기
		
		// 미리보기 CSS 추가
		$html .= '<style>
			#my-toc-live-preview .my-toc {
				margin: 0 auto !important;
				width: 100% !important;
				max-width: 400px !important;
				display: block !important;
				visibility: visible !important;
				opacity: 1 !important;
			}
			#my-toc-live-preview .my-toc-theme-gray {
				background-color: #f9f9f9 !important;
				border-color: #ddd !important;
			}
			#my-toc-live-preview .my-toc-theme-blue {
				background-color: #e6f6ff !important;
				border-color: #a8d8ff !important;
			}
			#my-toc-live-preview .my-toc-theme-white {
				background-color: #ffffff !important;
				border-color: #eeeeee !important;
			}
			#my-toc-live-preview .my-toc-theme-warm {
				background-color: #fff5e6 !important;
				border-color: #ffcc80 !important;
			}
			#my-toc-live-preview .active-link {
				font-weight: bold !important;
				background-color: rgba(0,0,0,0.05) !important;
				padding-left: 6px !important;
				border-left: 3px solid currentColor !important;
			}
		</style>';
		
		return $html;
	}	
	
	
	
    
    /**
     * 사용자 정의 색상 필드 콜백
     */
    public function custom_colors_callback() {
        $options = get_option('my_toc_options');
        $custom_colors = isset($options['custom_colors']) ? $options['custom_colors'] : array(
            'background' => '#f9f9f9',
            'border' => '#ddd',
            'title' => '#333',
            'link' => '#0073aa',
            'hover' => '#00a0d2',
            'visited' => '#800080'
        );
        
        // 각 색상 필드 정의
        $color_fields = array(
            'background' => array('배경색', 'bi-square-fill'),
            'border' => array('테두리 색상', 'bi-border-all'),
            'title' => array('제목 색상', 'bi-type-h2'),
            'link' => array('링크 색상', 'bi-link'),
            'hover' => array('마우스오버 색상', 'bi-mouse'),
            'visited' => array('방문한 링크 색상', 'bi-check-square')
        );
        
        echo '<div id="my-toc-custom-colors" style="margin-top: 10px;">';
        
        foreach ($color_fields as $field => $info) {
            $value = isset($custom_colors[$field]) ? $custom_colors[$field] : '';
            $label = $info[0];
            $icon = $info[1];
            ?>
            <div class="my-toc-color-field" style="margin-bottom: 10px;">
                <label for="my_toc_options_custom_colors_<?php echo esc_attr($field); ?>"><?php echo esc_html($label); ?></label>
                <div class="input-group">
                    <span class="input-group-text bg-white border-end-0">
                        <i class="bi <?php echo esc_attr($icon); ?> text-primary"></i>
                    </span>
                    <input 
                        type="color" 
                        id="my_toc_options_custom_colors_<?php echo esc_attr($field); ?>" 
                        name="my_toc_options[custom_colors][<?php echo esc_attr($field); ?>]" 
                        value="<?php echo esc_attr($value); ?>" 
                        class="form-control border-start-0"
                        placeholder="색상 선택"
                    >
                </div>
            </div>
            <?php
        }
        
        echo '</div>';
        ?>
        <p class="description">테마를 '사용자 정의'로 선택한 경우 사용할 색상을 설정합니다.</p>
        <?php
    }
    
    /**
     * 붙박이 기능 필드 콜백
     */
    public function sticky_callback() {
        $options = get_option('my_toc_options');
        $sticky = isset($options['sticky']) ? $options['sticky'] : 'off';
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-pin-angle text-primary"></i>
            </span>
            <select name="my_toc_options[sticky]" id="my-toc-sticky" class="form-control border-start-0" placeholder="붙박이 기능 설정">
                <option value="off" <?php selected($sticky, 'off'); ?>>끄기 (기본값)</option>
                <option value="on" <?php selected($sticky, 'on'); ?>>켜기</option>
            </select>
        </div>
        <p class="description">목차를 스크롤에 따라 고정 위치에 표시합니다.</p>
        <?php
    }
    
    /**
     * 붙박이 위치 필드 콜백
     */
    public function sticky_position_callback() {
        $options = get_option('my_toc_options');
        $sticky_position = isset($options['sticky_position']) ? $options['sticky_position'] : 'right-top';
        
        // 가능한 위치 정의
        $positions = array(
            'left-top' => '왼쪽 상단',
            'left-middle' => '왼쪽 중간',
            'left-bottom' => '왼쪽 하단',
            'right-top' => '오른쪽 상단 (기본값)',
            'right-middle' => '오른쪽 중간',
            'right-bottom' => '오른쪽 하단'
        );
        ?>
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-geo-alt text-primary"></i>
            </span>
            <select name="my_toc_options[sticky_position]" id="my-toc-sticky-position" class="form-control border-start-0" placeholder="붙박이 위치 선택">
                <?php foreach ($positions as $value => $label) : ?>
                    <option value="<?php echo esc_attr($value); ?>" <?php selected($sticky_position, $value); ?>>
                        <?php echo esc_html($label); ?>
                    </option>
                <?php endforeach; ?>
            </select>
        </div>
        <p class="description">붙박이 기능이 켜졌을 때 목차가 표시될 위치를 선택합니다.</p>
        <?php
    }
    
    /**
     * 옵션 유효성 검사
     */
    public function sanitize_options($input) {
        $sanitized = array();
        
        // 위치
        if (isset($input['position'])) {
            $sanitized['position'] = sanitize_text_field($input['position']);
        }
        
        // 최소 헤더 수
        if (isset($input['min_headers'])) {
            $sanitized['min_headers'] = absint($input['min_headers']);
            if ($sanitized['min_headers'] < 1) {
                $sanitized['min_headers'] = 2;
            }
        }
        
        // 헤더 라벨
        if (isset($input['header_label'])) {
            $sanitized['header_label'] = sanitize_text_field($input['header_label']);
        }
        
        // H2 접두사
        if (isset($input['h2_prefix'])) {
            $sanitized['h2_prefix'] = sanitize_text_field($input['h2_prefix']);
        }
        
        // H3 접두사
        if (isset($input['h3_prefix'])) {
            $sanitized['h3_prefix'] = sanitize_text_field($input['h3_prefix']);
        }
        
        // 초기 보기 상태
        if (isset($input['initial_view'])) {
            $sanitized['initial_view'] = sanitize_text_field($input['initial_view']);
        }
        
        // 테마
        if (isset($input['theme'])) {
            $sanitized['theme'] = sanitize_text_field($input['theme']);
        }
        
        // 사용자 정의 색상
        if (isset($input['custom_colors']) && is_array($input['custom_colors'])) {
            foreach ($input['custom_colors'] as $key => $value) {
                $sanitized['custom_colors'][$key] = sanitize_hex_color($value);
            }
        }
        
        // 붙박이 기능
        if (isset($input['sticky'])) {
            $sanitized['sticky'] = sanitize_text_field($input['sticky']);
        }
        
        // 붙박이 위치
        if (isset($input['sticky_position'])) {
            $sanitized['sticky_position'] = sanitize_text_field($input['sticky_position']);
        }
        
        return $sanitized;
    }
    
    /**
     * 메타박스 추가
     */
    public function add_toc_meta_box() {
        add_meta_box(
            'my_toc_meta_box',                 // ID
            '목차 설정',                        // 제목
            array($this, 'render_toc_meta_box'), // 콜백
            'post',                            // 화면 (post, page 등)
            'side',                            // 컨텍스트
            'default'                          // 우선순위
        );
    }
    
    /**
     * 메타박스 렌더링
     */
    public function render_toc_meta_box($post) {
        // nonce 필드 추가
        wp_nonce_field('my_toc_meta_box', 'my_toc_meta_box_nonce');
        
        // 현재 설정값 가져오기
        $disable_toc = get_post_meta($post->ID, '_my_toc_disable', true);
        ?>
        <p>
            <label for="my_toc_disable">
                <input type="checkbox" name="my_toc_disable" id="my_toc_disable" value="1" <?php checked($disable_toc, '1'); ?> />
                이 글에서 목차 비활성화
            </label>
        </p>
        <p class="description">이 옵션을 체크하면 이 글에서는 목차가 표시되지 않습니다.</p>
        <?php
    }
    
    /**
     * 메타박스 저장
     */
    public function save_toc_meta_box($post_id) {
        // 자동 저장 스킵
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        
        // nonce 검증
        if (!isset($_POST['my_toc_meta_box_nonce']) || !wp_verify_nonce($_POST['my_toc_meta_box_nonce'], 'my_toc_meta_box')) {
            return;
        }
        
        // 권한 확인
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }
        
        // 데이터 저장
        $disable_toc = isset($_POST['my_toc_disable']) ? '1' : '';
        update_post_meta($post_id, '_my_toc_disable', $disable_toc);
    }
    
    /**
     * 관리자 스크립트 및 스타일 등록
     */
    public function enqueue_admin_scripts($hook) {
        // 플러그인 설정 페이지에서만 로드
        if ('toplevel_page_my-toc-settings' !== $hook) {
            return;
        }
        
        // Bootstrap 5 CSS
        wp_enqueue_style(
            'bootstrap5',
            'https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css',
            array(),
            '5.3.0'
        );
        
        // Bootstrap Icons
        wp_enqueue_style(
            'bootstrap-icons',
            'https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.0/font/bootstrap-icons.css',
            array(),
            '1.11.0'
        );
        
        // 관리자 스타일
        wp_enqueue_style(
            'my-toc-admin',
            MY_TOC_PLUGIN_URL . 'assets/css/my-toc-admin.css',
            array('bootstrap5', 'bootstrap-icons'),
            MY_TOC_PLUGIN_VERSION
        );
        
        // jQuery (워드프레스에 내장됨)
        wp_enqueue_script('jquery');
        
        // Bootstrap 5 JS
        wp_enqueue_script(
            'bootstrap5',
            'https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js',
            array('jquery'),
            '5.3.0',
            true
        );
        
        // 관리자 스크립트
        wp_enqueue_script(
            'my-toc-admin',
            MY_TOC_PLUGIN_URL . 'assets/js/my-toc-admin.js',
            array('jquery', 'bootstrap5'),
            MY_TOC_PLUGIN_VERSION,
            true
        );
    }
}