<?php
/*
- 파일명: class-my-toc-frontend.php
- 위치: /my-toc-plugin/includes/
- 기능: TOC 플러그인의 프론트엔드 표시를 처리하는 클래스
- 작성일: 2025-03-23
- 수정일: 2025-03-25
*/

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

// ===================================
// 프론트엔드 클래스 정의
// ===================================
/**
 * TOC 플러그인의 프론트엔드 기능을 담당하는 클래스
 */
class My_TOC_Frontend {
    
    /**
     * 클래스 초기화
     *
     * @return void
     */
    public function init() {
        // 콘텐츠 필터 추가
        add_filter('the_content', array($this, 'add_toc_to_content'));
        
        // 스크립트 및 스타일 등록
        add_action('wp_enqueue_scripts', array($this, 'enqueue_frontend_scripts'));
    }
    
    // ===================================
    // 목차 생성 및 표시
    // ===================================
    /**
     * 콘텐츠에 TOC 추가
     *
     * @param string $content 원본 콘텐츠
     * @return string 목차가 추가된 콘텐츠
     */
    public function add_toc_to_content($content) {
        // 싱글 포스트가 아니면 원본 콘텐츠 반환
        if (!is_single()) {
            return $content;
        }
        
        // 현재 포스트 ID 가져오기
        $post_id = get_the_ID();
        
        // 이 포스트에서 TOC가 비활성화되었는지 확인
        $disable_toc = get_post_meta($post_id, '_my_toc_disable', true);
        if ($disable_toc == '1') {
            return $content;
        }
        
        // 플러그인 옵션 가져오기
        $options = get_option('my_toc_options', array());
        
        // 헤더 추출
        $headers = $this->extract_headers_from_content($content);
        
        // 최소 헤더 수 확인
        $min_headers = isset($options['min_headers']) ? intval($options['min_headers']) : 2;
        $h2_count = 0;
        
        foreach ($headers as $header) {
            if ($header['level'] == 2) {
                $h2_count++;
            }
        }
        
        // H2 태그가 최소 개수보다 적으면 원본 콘텐츠 반환
        if ($h2_count < $min_headers) {
            return $content;
        }
        
        // 헤더에 ID 추가
        $content = $this->add_header_ids_to_content($content, $headers);
        
        // 목차 HTML 생성
        $toc_html = $this->generate_toc_html($headers, $options);
        
        // 목차 위치 결정 및 삽입
        $position = isset($options['position']) ? $options['position'] : 'before';
        $content = $this->insert_toc_at_position($content, $toc_html, $position);
        
        return $content;
    }
    
    /**
     * 콘텐츠의 지정된 위치에 목차 삽입
     *
     * @param string $content 원본 콘텐츠
     * @param string $toc_html 목차 HTML
     * @param string $position 위치 (before 또는 after)
     * @return string 목차가 삽입된 콘텐츠
     */
    private function insert_toc_at_position($content, $toc_html, $position) {
        if ($position == 'before') {
            // 첫 머리글 앞에 삽입
            $pattern = '/<h[2-4][^>]*>/i';
            if (preg_match($pattern, $content, $matches, PREG_OFFSET_CAPTURE)) {
                $pos = $matches[0][1];
                $content = substr_replace($content, $toc_html, $pos, 0);
            } else {
                // 헤더가 없으면 콘텐츠 시작 부분에 추가
                $content = $toc_html . $content;
            }
        } else {
            // 첫 머리글 뒤에 삽입
            $pattern = '/<h[2-4][^>]*>.*?<\/h[2-4]>/i';
            if (preg_match($pattern, $content, $matches, PREG_OFFSET_CAPTURE)) {
                $pos = $matches[0][1] + strlen($matches[0][0]);
                $content = substr_replace($content, $toc_html, $pos, 0);
            } else {
                // 헤더가 없으면 콘텐츠 시작 부분에 추가
                $content = $toc_html . $content;
            }
        }
        
        return $content;
    }
    
    /**
     * 콘텐츠에서 헤더 추출
     *
     * @param string $content 콘텐츠
     * @return array 추출된 헤더 배열
     */
    private function extract_headers_from_content($content) {
        $headers = array();
        $pattern = '/<h([2-4])[^>]*>(.*?)<\/h\1>/is'; // 's' 플래그 추가로 개행 포함 일치
        
        if (preg_match_all($pattern, $content, $matches, PREG_SET_ORDER)) {
            foreach ($matches as $i => $match) {
                $level = intval($match[1]);
                $text = strip_tags($match[2]);
                $id = 'toc-' . sanitize_title($text) . '-' . $i;
                
                $headers[] = array(
                    'level' => $level,
                    'text' => $text,
                    'id' => $id,
                    'original' => $match[0],
                    'replacement' => '<h' . $level . ' id="' . $id . '">' . $match[2] . '</h' . $level . '>'
                );
            }
        }
        
        return $headers;
    }
    
    /**
     * 목차 HTML 생성
     *
     * @param array $headers 헤더 배열
     * @param array $options 플러그인 옵션
     * @return string 생성된 목차 HTML
     */
    private function generate_toc_html($headers, $options) {
        // 옵션 준비
        $header_label = isset($options['header_label']) ? esc_html($options['header_label']) : '목차';
        $initial_view = isset($options['initial_view']) ? $options['initial_view'] : 'show';
        $theme = isset($options['theme']) ? $options['theme'] : 'gray';
        $sticky = isset($options['sticky']) ? $options['sticky'] : 'off';
        $sticky_position = isset($options['sticky_position']) ? $options['sticky_position'] : 'right-top';
        $h2_prefix = isset($options['h2_prefix']) ? $options['h2_prefix'] : '';
        $h3_prefix = isset($options['h3_prefix']) ? $options['h3_prefix'] : '';
        
        // CSS 클래스 준비
        $toc_classes = array('my-toc');
        $toc_classes[] = 'my-toc-theme-' . $theme;
        
        // 붙박이 기능이 'on'일 때만 관련 클래스 추가
        if ($sticky === 'on') {
            $toc_classes[] = 'my-toc-sticky';
            $toc_classes[] = 'my-toc-sticky-' . $sticky_position;
        }
        
        if ($initial_view == 'hide') {
            $toc_classes[] = 'my-toc-collapsed';
        }
        
        // 사용자 정의 스타일 및 데이터 속성 준비
        $custom_style = '';
        $data_attrs = '';
        
        if ($theme == 'custom' && isset($options['custom_colors']) && is_array($options['custom_colors'])) {
            $custom_style = $this->generate_custom_style($options['custom_colors']);
            $data_attrs = $this->generate_data_attributes($options['custom_colors']);
            $toc_classes[] = 'my-toc-custom-colors';
        }
        
        // 목차 HTML 시작
        $html = $custom_style;
        $html .= '<div class="' . esc_attr(implode(' ', $toc_classes)) . '"' . $data_attrs . '>';
        
        // 헤더 및 토글 버튼
        $html .= '<div class="my-toc-header">';
        $html .= '<span class="my-toc-title">' . $header_label . '</span>';
        $html .= '<span class="my-toc-toggle" title="' . esc_attr('토글 목차') . '"><i class="bi ' . ($initial_view == 'hide' ? 'bi-plus' : 'bi-dash') . '"></i></span>';
        $html .= '</div>';
        
        // 목차 콘텐츠
        $html .= '<div class="my-toc-content' . ($initial_view == 'hide' ? ' my-toc-content-hidden' : '') . '">';
        $html .= $this->generate_toc_list($headers, $h2_prefix, $h3_prefix);
        $html .= '</div>'; // .my-toc-content 닫기
        $html .= '</div>'; // .my-toc 닫기
        
        return $html;
    }
    
    /**
     * 목차 리스트 HTML 생성
     *
     * @param array $headers 헤더 배열
     * @param string $h2_prefix H2 접두사
     * @param string $h3_prefix H3 접두사
     * @return string 생성된 목차 리스트 HTML
     */
    private function generate_toc_list($headers, $h2_prefix, $h3_prefix) {
        $html = '<ul class="my-toc-list">';
        
        $current_level = 2;
        $prev_level = 0;
        
        foreach ($headers as $header) {
            $level = $header['level'];
            
            // 레벨 차이에 따라 목록 열기/닫기
            if ($prev_level && $level > $prev_level) {
                // 새 하위 목록 시작
                $html .= '<ul class="my-toc-sublist">';
            } elseif ($prev_level && $level < $prev_level) {
                // 상위 수준으로 돌아가기
                $diff = $prev_level - $level;
                $html .= str_repeat('</li></ul>', $diff);
                $html .= '</li>';
            } elseif ($prev_level) {
                // 같은 레벨의 다음 항목
                $html .= '</li>';
            }
            
            // 목록 항목 시작
            $html .= '<li class="my-toc-item my-toc-level-' . $level . '">';
            
            // 헤더 레벨에 따라 접두사 추가
            $prefix = '';
            if ($level == 2 && !empty($h2_prefix)) {
                $prefix = '<span class="my-toc-prefix">' . esc_html($h2_prefix) . '</span>';
            } elseif ($level == 3 && !empty($h3_prefix)) {
                $prefix = '<span class="my-toc-prefix">' . esc_html($h3_prefix) . '</span>';
            }
            
            // 링크 생성 시 접두사와 텍스트 결합
            $html .= '<a href="#' . esc_attr($header['id']) . '">';
            if (!empty($prefix)) {
                $html .= $prefix;
            }
            $html .= esc_html($header['text']) . '</a>';
            
            $prev_level = $level;
        }
        
        // 열린 모든 목록 닫기
        if ($prev_level) {
            $diff = $prev_level - 2;
            $html .= str_repeat('</li></ul>', max(0, $diff));
            $html .= '</li>';
        }
        
        $html .= '</ul>'; // .my-toc-list 닫기
        
        return $html;
    }
    
    /**
     * 사용자 정의 색상을 위한 스타일 생성
     *
     * @param array $colors 색상 설정 배열
     * @return string 생성된 스타일 태그
     */
    private function generate_custom_style($colors) {
        $custom_style = '<style>';
        $custom_style .= '.my-toc.my-toc-theme-custom {';
        
        if (isset($colors['background'])) {
            $custom_style .= 'background-color: ' . esc_attr($colors['background']) . ' !important;';
        }
        
        if (isset($colors['border'])) {
            $custom_style .= 'border-color: ' . esc_attr($colors['border']) . ' !important;';
        }
        
        $custom_style .= '}';
        
        if (isset($colors['title'])) {
            $custom_style .= '.my-toc.my-toc-theme-custom .my-toc-title {';
            $custom_style .= 'color: ' . esc_attr($colors['title']) . ' !important;';
            $custom_style .= '}';
        }
        
        if (isset($colors['link'])) {
            $custom_style .= '.my-toc.my-toc-theme-custom .my-toc-list a {';
            $custom_style .= 'color: ' . esc_attr($colors['link']) . ' !important;';
            $custom_style .= '}';
        }
        
        if (isset($colors['hover'])) {
            $custom_style .= '.my-toc.my-toc-theme-custom .my-toc-list a:hover {';
            $custom_style .= 'color: ' . esc_attr($colors['hover']) . ' !important;';
            $custom_style .= 'background-color: ' . $this->adjust_color_opacity($colors['background'], 0.7) . ' !important;';
            $custom_style .= 'border-color: ' . esc_attr($colors['border']) . ' !important;';
            $custom_style .= '}';
        }
        
        if (isset($colors['visited'])) {
            $custom_style .= '.my-toc.my-toc-theme-custom .my-toc-list a:visited {';
            $custom_style .= 'color: ' . esc_attr($colors['visited']) . ' !important;';
            $custom_style .= '}';
        }
        
        $custom_style .= '</style>';
        
        return $custom_style;
    }
    
    /**
     * 사용자 정의 색상을 위한 데이터 속성 생성
     *
     * @param array $colors 색상 설정 배열
     * @return string 생성된 데이터 속성 문자열
     */
    private function generate_data_attributes($colors) {
        $link_color = isset($colors['link']) ? esc_attr($colors['link']) : '#0073aa';
        $hover_color = isset($colors['hover']) ? esc_attr($colors['hover']) : '#00a0d2';
        $visited_color = isset($colors['visited']) ? esc_attr($colors['visited']) : '#800080';
        $bg_color = isset($colors['background']) ? esc_attr($colors['background']) : '#f9f9f9';
        $border_color = isset($colors['border']) ? esc_attr($colors['border']) : '#ddd';
        
        $data_attrs = ' data-link-color="' . $link_color . '"';
        $data_attrs .= ' data-hover-color="' . $hover_color . '"';
        $data_attrs .= ' data-visited-color="' . $visited_color . '"';
        $data_attrs .= ' data-hover-bg="' . $this->adjust_color_opacity($bg_color, 0.7) . '"';
        $data_attrs .= ' data-border-color="' . $border_color . '"';
        
        return $data_attrs;
    }
    
    /**
     * 색상의 투명도를 조정하는 함수
     *
     * @param string $color 헥스 색상 코드
     * @param float $opacity 투명도 (0~1 사이)
     * @return string rgba 색상 문자열
     */
    private function adjust_color_opacity($color, $opacity) {
        // #으로 시작하는 헥스 코드인지 확인
        if (substr($color, 0, 1) == '#') {
            $color = substr($color, 1);
        }
        
        // 색상 길이에 따라 처리
        if (strlen($color) == 6) {
            $hex = array($color[0].$color[1], $color[2].$color[3], $color[4].$color[5]);
        } elseif (strlen($color) == 3) {
            $hex = array($color[0].$color[0], $color[1].$color[1], $color[2].$color[2]);
        } else {
            return 'rgba(0,0,0,' . $opacity . ')';
        }
        
        // RGB로 변환
        $rgb = array_map('hexdec', $hex);
        
        // RGBA 문자열 반환
        return 'rgba(' . implode(',', $rgb) . ',' . $opacity . ')';
    }
    
    /**
     * 콘텐츠에 헤더 ID 추가
     *
     * @param string $content 원본 콘텐츠
     * @param array $headers 헤더 배열
     * @return string ID가 추가된 콘텐츠
     */
    private function add_header_ids_to_content($content, $headers) {
        foreach ($headers as $header) {
            $content = str_replace($header['original'], $header['replacement'], $content);
        }
        
        return $content;
    }
    
    // ===================================
    // 스크립트 및 스타일
    // ===================================
    /**
     * 프론트엔드 스크립트 및 스타일 등록
     *
     * @return void
     */
    public function enqueue_frontend_scripts() {
        // 싱글 포스트에서만 로드
        if (!is_single()) {
            return;
        }
        
        // 현재 포스트 ID 가져오기
        $post_id = get_the_ID();
        
        // 이 포스트에서 TOC가 비활성화되었는지 확인
        $disable_toc = get_post_meta($post_id, '_my_toc_disable', true);
        if ($disable_toc == '1') {
            return;
        }
        
        // Bootstrap Icons
        wp_enqueue_style(
            'bootstrap-icons',
            'https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.0/font/bootstrap-icons.css',
            array(),
            '1.11.0'
        );
        
        // 프론트엔드 스타일
        wp_enqueue_style(
            'my-toc-frontend',
            MY_TOC_PLUGIN_URL . 'assets/css/my-toc-frontend.css',
            array('bootstrap-icons'),
            MY_TOC_PLUGIN_VERSION . '.' . time(), // 현재 시간을 버전에 추가하여 캐시 방지
            'all'
        );
        
        // jQuery (워드프레스에 내장됨)
        wp_enqueue_script('jquery');
        
        // 프론트엔드 스크립트
        wp_enqueue_script(
            'my-toc-frontend',
            MY_TOC_PLUGIN_URL . 'assets/js/my-toc-frontend.js',
            array('jquery'),
            MY_TOC_PLUGIN_VERSION . '.' . time(), // 현재 시간을 버전에 추가하여 캐시 방지
            true
        );
        
        // 플러그인 옵션 가져오기
        $options = get_option('my_toc_options', array());
        
        // 스크립트에 옵션 전달
        wp_localize_script(
            'my-toc-frontend',
            'myTocOptions',
            array(
                'initialView' => isset($options['initial_view']) ? $options['initial_view'] : 'show',
                'sticky' => isset($options['sticky']) ? $options['sticky'] : 'off',
                'stickyPosition' => isset($options['sticky_position']) ? $options['sticky_position'] : 'right-top',
            )
        );
    }
}